import { Observable } from 'rxjs/Observable';
import { AwgInstrument } from './instruments/awg/awg-instrument';
import { DcInstrument } from './instruments/dc/dc-instrument';
import { LaInstrument } from './instruments/la/la-instrument';
import { OscInstrument } from './instruments/osc/osc-instrument';
import { TriggerInstrument } from './instruments/trigger/trigger-instrument';
import { GpioInstrument } from './instruments/gpio/gpio-instrument';
import { LoggerInstrument } from './instruments/logger/logger-instrument';
import { File } from './file';
import { TransportContainer } from '../transport/transport-container';
export declare class Device {
    transport: TransportContainer;
    descriptorObject: any;
    rootUri: string;
    deviceMake: string;
    deviceModel: string;
    firmwareVersion: any;
    calibrationSource: 'UNCALIBRATED' | 'CALIBRATED' | 'FACTORY' | 'USER';
    macAddress: string;
    instruments: {
        awg: AwgInstrument;
        dc: DcInstrument;
        la: LaInstrument;
        osc: OscInstrument;
        trigger: TriggerInstrument;
        gpio: GpioInstrument;
        logger: LoggerInstrument;
    };
    file: File;
    firmwareRepositoryUrl: string;
    listFirmwareUrl: string;
    private httpTimeout;
    constructor(_rootUri: string, deviceDescriptor: any, httpTimeout: number);
    multiCommand(commandObject: any): Observable<any>;
    _genericResponseHandler(commandObject: Object): Observable<any>;
    resetInstruments(): Observable<any>;
    storageGetLocations(): Observable<any>;
    calibrationGetStorageTypes(): Observable<any>;
    calibrationGetInstructions(): Observable<any>;
    calibrationStart(): Observable<any>;
    calibrationLoad(type: string): Observable<any>;
    calibrationRead(): Observable<any>;
    calibrationSave(type: string): Observable<any>;
    calibrationGetStatus(): Observable<any>;
    nicList(): Observable<any>;
    nicGetStatus(adapter: string): Observable<any>;
    nicConnect(adapter: string, parameterSet: 'activeParameterSet' | 'workingParameterSet', force: boolean): Observable<any>;
    nicDisconnect(adapter: string): Observable<any>;
    wifiScan(adapter: string): Observable<any>;
    wifiReadScannedNetworks(adapter: string): Observable<any>;
    wifiSetParameters(adapter: string, ssid: string, securityType: 'wep40' | 'wep104' | 'wpa' | 'wpa2' | 'open', autoConnect: boolean, passphrase?: string, keys?: string, keyIndex?: number): Observable<any>;
    wifiListSavedParameters(storageLocation: string): Observable<any>;
    wifiDeleteParameters(storageLocation: string, ssid: string): Observable<any>;
    wifiSaveParameters(storageLocation: string): Observable<any>;
    wifiLoadParameters(storageLocation: string, ssid: string): Observable<any>;
}
