"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Observable_1 = require("rxjs/Observable");
var command_utility_1 = require("../utilities/command-utility");
var File = /** @class */ (function () {
    function File(transport) {
        this.transport = transport;
        this.commandUtil = new command_utility_1.CommandUtility();
    }
    //Set the output voltage of the specified DC power supply channel.
    File.prototype.write = function (location, path, file, filePosition) {
        var _this = this;
        filePosition = filePosition == undefined ? 0 : filePosition;
        var command = {
            file: [{
                    command: 'write',
                    type: location,
                    path: path,
                    filePosition: filePosition,
                    binaryOffset: 0,
                    binaryLength: file.byteLength
                }]
        };
        return Observable_1.Observable.create(function (observer) {
            var dataToSend = _this.commandUtil.createChunkedArrayBuffer(command, file);
            _this.transport.writeRead('/', dataToSend, 'binary').subscribe(function (arrayBuffer) {
                var data;
                try {
                    var stringify = String.fromCharCode.apply(null, new Uint8Array(arrayBuffer.slice(0)));
                    data = JSON.parse(stringify);
                }
                catch (e) {
                    observer.error(e);
                    return;
                }
                if (data == undefined || data.agent != undefined) {
                    observer.error(data);
                    return;
                }
                if (data.file == undefined || data.file[0] == undefined || data.file[0].statusCode !== 0) {
                    observer.error(data);
                    return;
                }
                observer.next(data);
                observer.complete();
            }, function (err) {
                observer.error(err);
            }, function () {
                observer.complete();
            });
        });
    };
    File.prototype.listDir = function (location, path) {
        var command = {
            file: [{
                    command: 'listdir',
                    type: location,
                    path: path
                }]
        };
        return this.genericResponse(command);
    };
    File.prototype.getFileSize = function (location, path) {
        var command = {
            file: [{
                    command: 'getFileSize',
                    type: location,
                    path: path
                }]
        };
        return this.genericResponse(command);
    };
    File.prototype.delete = function (location, path) {
        var command = {
            file: [{
                    command: 'delete',
                    type: location,
                    path: path
                }]
        };
        return this.genericResponse(command);
    };
    //Set the output voltage of the specified DC power supply channel.
    File.prototype.read = function (location, path, filePosition, length, timeoutOverride) {
        var _this = this;
        var command = {
            file: [{
                    command: 'read',
                    type: location,
                    path: path,
                    filePosition: filePosition,
                    requestedLength: length
                }]
        };
        return Observable_1.Observable.create(function (observer) {
            _this.transport.writeRead('/', JSON.stringify(command), 'json', timeoutOverride).subscribe(function (arrayBuffer) {
                _this.commandUtil.observableParseChunkedTransfer(arrayBuffer, 'u8').subscribe(function (data) {
                    var jsonObject = data.json;
                    var binaryData = data.typedArray;
                    if (data == undefined || data.agent != undefined) {
                        observer.error(data);
                        return;
                    }
                    if (jsonObject.file == undefined || jsonObject.file[0] == undefined || jsonObject.file[0].statusCode !== 0) {
                        observer.error(data);
                        return;
                    }
                    observer.next({
                        jsonObject: jsonObject,
                        file: _this.safeStringBinary(binaryData)
                    });
                    observer.complete();
                }, function (err) {
                    observer.error(err);
                }, function () { });
            }, function (err) {
                observer.error(err);
            }, function () {
                observer.complete();
            });
        });
    };
    File.prototype.safeStringBinary = function (binaryData) {
        var returnString = '';
        for (var i = 0; i < binaryData.length; i++) {
            returnString += String.fromCharCode(binaryData[i]);
        }
        return returnString;
    };
    File.prototype.genericResponse = function (command) {
        var _this = this;
        return Observable_1.Observable.create(function (observer) {
            _this.transport.writeRead('/', JSON.stringify(command), 'json').subscribe(function (arrayBuffer) {
                var data;
                try {
                    var stringify = String.fromCharCode.apply(null, new Uint8Array(arrayBuffer.slice(0)));
                    data = JSON.parse(stringify);
                }
                catch (e) {
                    observer.error(e);
                    return;
                }
                if (data == undefined || data.agent != undefined) {
                    observer.error(data);
                    return;
                }
                if (data.file == undefined || data.file[0] == undefined || data.file[0].statusCode !== 0) {
                    observer.error(data);
                    return;
                }
                observer.next(data);
                observer.complete();
            }, function (err) {
                observer.error(err);
            }, function () {
                observer.complete();
            });
        });
    };
    return File;
}());
exports.File = File;
//# sourceMappingURL=file.js.map