"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Observable_1 = require("rxjs/Observable");
var generic_instrument_1 = require("../generic-instrument");
var awg_channel_1 = require("./awg-channel");
var AwgInstrument = /** @class */ (function (_super) {
    __extends(AwgInstrument, _super);
    function AwgInstrument(_transport, _awgInstrumentDescriptor) {
        var _this = _super.call(this, _transport, '/') || this;
        _this.chans = [];
        //Populate AWG supply parameters
        _this.numChans = _awgInstrumentDescriptor.numChans;
        //Populate channels  
        for (var key in _awgInstrumentDescriptor) {
            if (key !== 'numChans') {
                _this.chans.push(new awg_channel_1.AwgChannel(_awgInstrumentDescriptor[key]));
            }
        }
        return _this;
    }
    AwgInstrument.prototype.setArbitraryWaveform = function (chans, waveforms, dataTypes) {
        var _this = this;
        var command = {
            "awg": {}
        };
        var binaryOffset = 0;
        var binaryLength = 0;
        chans.forEach(function (element, index, array) {
            //Hard code 2 bytes per value
            //TODO read data types and generate length from that instead of hard code
            binaryLength = waveforms[index].y.length * 2;
            command.awg[chans[index]] =
                [
                    {
                        command: "setArbitraryWaveform",
                        binaryOffset: binaryOffset,
                        binaryLength: binaryLength,
                        binaryType: dataTypes[index],
                        vpp: 3,
                        vOffset: 0,
                        dt: waveforms[index].dt
                    }
                ];
            binaryOffset += binaryLength;
        });
        var stringCommand = JSON.stringify(command) + '\r\n';
        var jsonChars = stringCommand.length;
        var fullString = jsonChars + '\r\n' + stringCommand;
        var binaryBufferToSend = new ArrayBuffer(fullString.length + binaryOffset);
        for (var i = 0; i < fullString.length; i++) {
            binaryBufferToSend[i] = fullString.charCodeAt(i);
        }
        for (var i = 0; i < chans.length; i++) {
            var typedArray = new Int16Array(waveforms[i].y);
            var byteConvert = new Uint8Array(typedArray.buffer);
            for (var i_1 = fullString.length, j = 0; i_1 < binaryOffset + fullString.length; i_1 = i_1 + 2, j = j + 2) {
                binaryBufferToSend[i_1] = byteConvert[j];
                binaryBufferToSend[i_1 + 1] = byteConvert[j + 1];
            }
        }
        return Observable_1.Observable.create(function (observer) {
            _this.transport.writeRead(_this.endpoint, binaryBufferToSend, 'binary').subscribe(function (arrayBuffer) {
                var data = JSON.parse(String.fromCharCode.apply(null, new Int8Array(arrayBuffer.slice(0))));
                for (var i = 0; i < chans.length; i++) {
                    if (data.awg == undefined || data.awg[chans[i]][0].statusCode > 0 || data.agent != undefined) {
                        observer.error(data);
                        return;
                    }
                }
                observer.next(data);
                observer.complete();
            }, function (err) {
                observer.error(err);
            }, function () {
                observer.complete();
            });
        });
    };
    AwgInstrument.prototype.getCurrentState = function (chans) {
        var command = {
            awg: {}
        };
        chans.forEach(function (element, index, array) {
            command.awg[chans[index]] =
                [{
                        command: "getCurrentState"
                    }];
        });
        return _super.prototype._genericResponseHandler.call(this, command);
    };
    AwgInstrument.prototype.setRegularWaveform = function (chans, settings) {
        var command = {
            "awg": {}
        };
        chans.forEach(function (element, index, array) {
            command.awg[chans[index]] =
                [
                    {
                        command: "setRegularWaveform",
                        signalType: settings[index].signalType,
                        signalFreq: Math.floor(settings[index].signalFreq * 1000),
                        vpp: Math.floor(settings[index].vpp * 1000),
                        vOffset: Math.floor(settings[index].vOffset * 1000)
                    }
                ];
        });
        return _super.prototype._genericResponseHandler.call(this, command);
    };
    AwgInstrument.prototype.setRegularWaveformJson = function (chans, settings) {
        var command = {
            "awg": {}
        };
        chans.forEach(function (element, index, array) {
            command.awg[chans[index]] =
                [
                    {
                        command: "setRegularWaveform",
                        signalType: settings[index].signalType,
                        signalFreq: Math.floor(settings[index].signalFreq * 1000),
                        vpp: Math.floor(settings[index].vpp * 1000),
                        vOffset: Math.floor(settings[index].vOffset * 1000)
                    }
                ];
        });
        return command;
    };
    AwgInstrument.prototype.setRegularWaveformParse = function (chan, responseObject) {
        return responseObject;
    };
    AwgInstrument.prototype.runJson = function (chans) {
        var command = {
            "awg": {}
        };
        chans.forEach(function (element, index, array) {
            command.awg[chans[index]] =
                [
                    {
                        command: "run"
                    }
                ];
        });
        return command;
    };
    AwgInstrument.prototype.runParse = function (chan, responseObject) {
        return responseObject;
    };
    AwgInstrument.prototype.run = function (chans) {
        var command = {
            "awg": {}
        };
        chans.forEach(function (element, index, array) {
            command.awg[chans[index]] =
                [
                    {
                        command: "run"
                    }
                ];
        });
        return _super.prototype._genericResponseHandler.call(this, command);
    };
    AwgInstrument.prototype.stop = function (chans) {
        var command = {
            "awg": {}
        };
        chans.forEach(function (element, index, array) {
            command.awg[chans[index]] =
                [
                    {
                        command: "stop"
                    }
                ];
        });
        return _super.prototype._genericResponseHandler.call(this, command);
    };
    return AwgInstrument;
}(generic_instrument_1.GenericInstrument));
exports.AwgInstrument = AwgInstrument;
//# sourceMappingURL=awg-instrument.js.map