"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Observable_1 = require("rxjs/Observable");
var generic_instrument_1 = require("../generic-instrument");
var GpioInstrument = /** @class */ (function (_super) {
    __extends(GpioInstrument, _super);
    function GpioInstrument(_transport, _gpioInstrumentDescriptor) {
        var _this = _super.call(this, _transport, '/') || this;
        //Populate LA supply parameters
        _this.numChans = _gpioInstrumentDescriptor.numChans;
        _this.sinkCurrentMax = _gpioInstrumentDescriptor.sinkCurrentMax;
        _this.sourceCurrentMax = _gpioInstrumentDescriptor.sourceCurrentMax;
        return _this;
    }
    GpioInstrument.prototype.setParameters = function (chans, directions) {
        var command = {
            "gpio": {}
        };
        chans.forEach(function (element, index, array) {
            command.gpio[chans[index]] =
                [
                    {
                        "command": "setParameters",
                        "direction": directions[index]
                    }
                ];
        });
        return _super.prototype._genericResponseHandler.call(this, command);
    };
    //Set the output voltage of the specified DC power supply channel.
    GpioInstrument.prototype.write = function (chans, values) {
        var command = {
            "gpio": {}
        };
        values.forEach(function (element, index, array) {
            command.gpio[chans[index]] =
                [
                    {
                        "command": "write",
                        "value": values[index]
                    }
                ];
        });
        return _super.prototype._genericResponseHandler.call(this, command);
    };
    //Set the output voltage of the specified DC power supply channel.
    GpioInstrument.prototype.read = function (chans) {
        var _this = this;
        var command = {
            "gpio": {}
        };
        chans.forEach(function (element, index, array) {
            command.gpio[chans[index]] =
                [
                    {
                        "command": "read"
                    }
                ];
        });
        return Observable_1.Observable.create(function (observer) {
            if (chans.length < 1) {
                observer.error('No Channels Specified');
            }
            _this.transport.writeRead(_this.endpoint, JSON.stringify(command), 'json').subscribe(function (arrayBuffer) {
                //Handle device errors and warnings
                var data;
                try {
                    data = JSON.parse(String.fromCharCode.apply(null, new Int8Array(arrayBuffer.slice(0))));
                }
                catch (e) {
                    observer.error(e);
                    return;
                }
                for (var i = 0; i < chans.length; i++) {
                    if (data.gpio == undefined || data.gpio[chans[i]][0].statusCode > 0 || data.agent != undefined) {
                        observer.error(data);
                        return;
                    }
                }
                observer.next(data);
                observer.complete();
            }, function (err) {
                observer.error(err);
            }, function () {
                observer.complete();
            });
        });
    };
    return GpioInstrument;
}(generic_instrument_1.GenericInstrument));
exports.GpioInstrument = GpioInstrument;
//# sourceMappingURL=gpio-instrument.js.map