"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Observable_1 = require("rxjs/Observable");
require("rxjs/add/operator/mergeMap");
var command_utility_1 = require("../../../utilities/command-utility");
var LoggerCommand = /** @class */ (function () {
    function LoggerCommand(_loggerInstrumentRef) {
        this.instrumentRef = _loggerInstrumentRef;
        this.commandUtility = new command_utility_1.CommandUtility();
    }
    LoggerCommand.prototype.analogSetParametersJson = function (chans, maxSampleCounts, gains, vOffsets, sampleFreqs, startDelays, overflows, storageLocations, uris) {
        var command = {
            "log": {
                "analog": {}
            }
        };
        chans.forEach(function (element, index, array) {
            command.log.analog[chans[index]] =
                [
                    {
                        "command": "setParameters",
                        "maxSampleCount": maxSampleCounts[index],
                        "gain": gains[index],
                        "vOffset": Math.round(vOffsets[index] * 1000),
                        "sampleFreq": Math.round(sampleFreqs[index] * 1000000),
                        "startDelay": Math.round(startDelays[index] * Math.pow(10, 12)),
                        "overflow": overflows[index],
                        "storageLocation": storageLocations[index],
                        "uri": uris[index]
                    }
                ];
        });
        return command;
    };
    LoggerCommand.prototype.analogSetParametersParse = function (chan, responseObject) {
        return 'Channel ' + chan + ' ' + responseObject.command + ' successful';
    };
    LoggerCommand.prototype.digitalSetParametersParse = function (chan, responseObject) {
        return 'Channel ' + chan + ' ' + responseObject.command + ' successful';
    };
    LoggerCommand.prototype.runParse = function (chan, responseObject) {
        return 'Channel ' + chan + ' ' + responseObject.command + ' successful';
    };
    LoggerCommand.prototype.stopParse = function (chan, responseObject) {
        return 'Channel ' + chan + ' ' + responseObject.command + ' successful';
    };
    LoggerCommand.prototype.digitalSetParametersJson = function (chans, maxSampleCounts, sampleFreqs, startDelays, overflows, storageLocations, uris, bitMasks) {
        var command = {
            "log": {
                "digital": {}
            }
        };
        chans.forEach(function (element, index, array) {
            command.log.digital[chans[index]] =
                [
                    {
                        "command": "setParameters",
                        "bitMask": bitMasks[index],
                        "maxSampleCount": maxSampleCounts[index],
                        "sampleFreq": Math.round(sampleFreqs[index] * 1000000),
                        "startDelay": Math.round(startDelays[index] * Math.pow(10, 12)),
                        "overflow": overflows[index],
                        "storageLocation": storageLocations[index],
                        "uri": uris[index]
                    }
                ];
        });
        return command;
    };
    LoggerCommand.prototype.runJson = function (instrument, chans) {
        var command = {
            "log": {}
        };
        command.log[instrument] = {};
        chans.forEach(function (element, index, array) {
            command.log[instrument][chans[index]] =
                [
                    {
                        command: "run"
                    }
                ];
        });
        return command;
    };
    LoggerCommand.prototype.stopJson = function (instrument, chans) {
        var command = {
            "log": {}
        };
        command.log[instrument] = {};
        chans.forEach(function (element, index, array) {
            command.log[instrument][chans[index]] =
                [
                    {
                        command: "stop"
                    }
                ];
        });
        return command;
    };
    LoggerCommand.prototype.readJson = function (instrument, chans, startIndices, counts) {
        var command = {
            "log": {}
        };
        command.log[instrument] = {};
        chans.forEach(function (element, index, array) {
            command.log[instrument][chans[index]] =
                [
                    {
                        command: "read",
                        startIndex: startIndices[index],
                        count: counts[index]
                    }
                ];
        });
        return command;
    };
    LoggerCommand.prototype.getCurrentStateJson = function (instrument, chans) {
        var command = {
            "log": {}
        };
        command.log[instrument] = {};
        chans.forEach(function (element, index, array) {
            command.log[instrument][chans[index]] =
                [
                    {
                        command: "getCurrentState"
                    }
                ];
        });
        return command;
    };
    LoggerCommand.prototype.analogSetParameters = function (chans, maxSampleCounts, gains, vOffsets, sampleFreqs, startDelays, overflows, storageLocations, uris) {
        var command = this.analogSetParametersJson(chans, maxSampleCounts, gains, vOffsets, sampleFreqs, startDelays, overflows, storageLocations, uris);
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommand.prototype.digitalSetParameters = function (chans, maxSampleCounts, sampleFreqs, startDelays, overflows, storageLocations, uris, bitMasks) {
        var command = this.digitalSetParametersJson(chans, maxSampleCounts, sampleFreqs, startDelays, overflows, storageLocations, uris, bitMasks);
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommand.prototype.run = function (instrument, chans) {
        var command = this.runJson(instrument, chans);
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommand.prototype.stop = function (instrument, chans) {
        var command = this.stopJson(instrument, chans);
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommand.prototype.read = function (instrument, chans, startIndices, counts) {
        var _this = this;
        var command = this.readJson(instrument, chans, startIndices, counts);
        return Observable_1.Observable.create(function (observer) {
            _this.instrumentRef.transport.writeRead('/', JSON.stringify(command), 'binary')
                .flatMap(function (data) {
                return _this.commandUtility.observableParseChunkedTransfer(data);
            })
                .subscribe(function (data) {
                var returnObject = {
                    cmdRespObj: data.json,
                    instruments: {}
                };
                var command = data.json;
                console.log(command);
                for (var instrument_1 in command.log) {
                    returnObject.instruments[instrument_1] = {};
                    for (var channel in command.log[instrument_1]) {
                        returnObject.instruments[instrument_1][channel] = {};
                        if (command.log[instrument_1][channel][0].statusCode > 0) {
                            observer.error('StatusCode error: ' + instrument_1 + ' Ch ' + channel);
                            return;
                        }
                        if (command.log[instrument_1][channel][0].binaryLength === 0) {
                            observer.error('No data received on ' + instrument_1 + ' Ch ' + channel);
                            return;
                        }
                        var binaryOffset = command.log[instrument_1][channel][0].binaryOffset / 2;
                        var binaryData = data.typedArray.slice(binaryOffset, binaryOffset + command.log[instrument_1][channel][0].binaryLength / 2);
                        var untypedArray = Array.prototype.slice.call(binaryData);
                        var scaledArray = untypedArray.map(function (voltage) {
                            return voltage / 1000;
                        });
                        Object.assign(returnObject.instruments[instrument_1][channel], command.log[instrument_1][channel][0]);
                        returnObject.instruments[instrument_1][channel]['data'] = scaledArray;
                    }
                }
                observer.next(returnObject);
                observer.complete();
            }, function (err) {
                console.log(err);
                observer.error(err);
            }, function () { });
        });
    };
    LoggerCommand.prototype.getCurrentState = function (instrument, chans) {
        var command = this.getCurrentStateJson(instrument, chans);
        return this.instrumentRef._genericResponseHandler(command);
    };
    return LoggerCommand;
}());
exports.LoggerCommand = LoggerCommand;
//# sourceMappingURL=logger-command.js.map