"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Observable_1 = require("rxjs/Observable");
var generic_instrument_1 = require("../generic-instrument");
var osc_channel_1 = require("./osc-channel");
var waveform_1 = require("../../../data-types/waveform");
var command_utility_1 = require("../../../utilities/command-utility");
var OscInstrument = /** @class */ (function (_super) {
    __extends(OscInstrument, _super);
    function OscInstrument(_transport, _oscInstrumentDescriptor) {
        var _this = _super.call(this, _transport, '/') || this;
        _this.chans = [];
        _this.numDataBuffers = 8;
        _this.dataBuffer = [];
        _this.dataBufferWriteIndex = 0;
        _this.dataBufferReadIndex = 0;
        //Populate DC supply parameters
        _this.numChans = _oscInstrumentDescriptor.numChans;
        _this.commandUtility = new command_utility_1.CommandUtility();
        //Populate channels        
        for (var channel in _oscInstrumentDescriptor) {
            if (channel !== 'numChans') {
                _this.chans.push(new osc_channel_1.OscChannel(_oscInstrumentDescriptor[channel]));
            }
        }
        for (var i = 0; i < _this.numDataBuffers; i++) {
            _this.dataBuffer.push([]);
        }
        return _this;
    }
    OscInstrument.prototype.getCurrentState = function (chans) {
        var command = this.getCurrentStateJson(chans);
        return _super.prototype._genericResponseHandler.call(this, command);
    };
    OscInstrument.prototype.getCurrentStateJson = function (chans) {
        var command = {
            osc: {}
        };
        chans.forEach(function (element, index, array) {
            command.osc[chans[index]] =
                [
                    {
                        command: 'getCurrentState'
                    }
                ];
        });
        return command;
    };
    OscInstrument.prototype.getCurrentStateParse = function (chan, responseObject) {
        return 'Success';
    };
    OscInstrument.prototype.setParametersJson = function (chans, offsets, gains, sampleFreqs, bufferSizes, delays) {
        var command = {
            "osc": {}
        };
        chans.forEach(function (element, index, array) {
            command.osc[chans[index]] =
                [
                    {
                        command: "setParameters",
                        vOffset: Math.round(offsets[index] * 1000),
                        gain: gains[index],
                        sampleFreq: Math.round(sampleFreqs[index] * 1000),
                        bufferSize: Math.round(bufferSizes[index]),
                        triggerDelay: Math.round(delays[index] * 1000000000000)
                    }
                ];
        });
        return command;
    };
    OscInstrument.prototype.setParametersParse = function (chan, responseObject) {
        return 'Channel ' + chan + ' ' + responseObject.command + ' successful';
    };
    //Tell OpenScope to run once and return a buffer
    OscInstrument.prototype.setParameters = function (chans, offsets, gains, sampleFreqs, bufferSizes, delays) {
        if (chans.length == 0) {
            return Observable_1.Observable.create(function (observer) {
                observer.complete();
            });
        }
        var command = this.setParametersJson(chans, offsets, gains, sampleFreqs, bufferSizes, delays);
        return _super.prototype._genericResponseHandler.call(this, command);
    };
    //Tell OpenScope to run once and return a buffer
    OscInstrument.prototype.read = function (chans) {
        var _this = this;
        if (chans.length == 0) {
            return Observable_1.Observable.create(function (observer) {
                observer.complete();
            });
        }
        var command = {
            "osc": {}
        };
        chans.forEach(function (element, index, array) {
            command.osc[chans[index]] =
                [
                    {
                        "command": "read"
                    }
                ];
        });
        this.dataBuffer[this.dataBufferWriteIndex] = [];
        return Observable_1.Observable.create(function (observer) {
            _this.transport.writeRead('/', JSON.stringify(command), 'json').subscribe(function (data) {
                _this.rawPacket = data;
                _this.commandUtility.observableParseChunkedTransfer(data).subscribe(function (data) {
                    var command = data.json;
                    console.log(command);
                    for (var channel in command.osc) {
                        if (command.osc[channel][0].statusCode > 0) {
                            observer.error('One or more channels still acquiring');
                            return;
                        }
                        var binaryOffset = command.osc[channel][0].binaryOffset / 2;
                        var binaryData = data.typedArray.slice(binaryOffset, binaryOffset + command.osc[channel][0].binaryLength / 2);
                        var untypedArray = Array.prototype.slice.call(binaryData);
                        var scaledArray = untypedArray.map(function (voltage) {
                            return voltage / 1000;
                        });
                        var dt = 1 / (command.osc[channel][0].actualSampleFreq / 1000);
                        var pointContainer = [];
                        var triggerPosition = -1 * command.osc[channel][0].triggerDelay / Math.pow(10, 12) + dt * scaledArray.length / 2;
                        for (var i = 0; i < scaledArray.length; i++) {
                            pointContainer.push([i * dt - triggerPosition, scaledArray[i]]);
                        }
                        _this.dataBuffer[_this.dataBufferWriteIndex][parseInt(channel) - 1] = new waveform_1.Waveform({
                            dt: 1 / (command.osc[channel][0].actualSampleFreq / 1000),
                            t0: 0,
                            y: scaledArray,
                            data: pointContainer,
                            pointOfInterest: command.osc[channel][0].pointOfInterest,
                            triggerPosition: command.osc[channel][0].triggerIndex,
                            seriesOffset: command.osc[channel][0].actualVOffset / 1000,
                            triggerDelay: (command.osc[channel][0].triggerDelay == undefined ? command.osc[channel][0].actualTriggerDelay : command.osc[channel][0].triggerDelay)
                        });
                    }
                    _this.dataBufferReadIndex = _this.dataBufferWriteIndex;
                    _this.dataBufferWriteIndex = (_this.dataBufferWriteIndex + 1) % _this.numDataBuffers;
                    var finish = performance.now();
                    observer.next(command);
                    //Handle device errors and warnings
                    observer.complete();
                }, function (err) {
                    observer.error(data);
                }, function () { });
            }, function (err) {
                observer.error(err);
            }, function () { });
        });
    };
    return OscInstrument;
}(generic_instrument_1.GenericInstrument));
exports.OscInstrument = OscInstrument;
//# sourceMappingURL=osc-instrument.js.map