"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var SimulatedLa = /** @class */ (function () {
    function SimulatedLa(_simulatedDevice) {
        this.buffers = [];
        this.sampleFreqs = [];
        this.bufferSizes = [];
        this.bitmasks = [];
        this.triggerDelays = [];
        this.simulatedDevice = _simulatedDevice;
        this.laDescriptor = this.simulatedDevice.getEnumeration().la;
        for (var i = 0; i < this.laDescriptor.numChans; i++) {
            this.buffers.push([]);
            this.sampleFreqs.push(0);
            this.bufferSizes.push(0);
            this.bitmasks.push(0);
            this.triggerDelays.push(0);
        }
    }
    SimulatedLa.prototype.getCurrentState = function (chan) {
        return {
            command: "getCurrentState",
            statusCode: 0,
            state: "idle",
            acqCount: 0,
            bitmask: this.bitmasks[chan],
            actualSampleFreq: this.sampleFreqs[chan],
            actualBufferSize: this.bufferSizes[chan],
            triggerDelay: this.triggerDelays[chan],
            wait: 0
        };
    };
    SimulatedLa.prototype.setParameters = function (chan, commandObject) {
        this.sampleFreqs[chan] = commandObject.sampleFreq;
        this.bufferSizes[chan] = commandObject.bufferSize;
        this.triggerDelays[chan] = commandObject.triggerDelay;
        this.bitmasks[chan] = commandObject.bitmask;
        this.simulatedDevice.setLaParameters(commandObject, chan);
        return {
            "command": "setParameters",
            "statusCode": 0,
            "actualSampleFreq": this.sampleFreqs[chan],
            "wait": 0
        };
    };
    SimulatedLa.prototype.read = function (chan) {
        return this.generateLaData(chan);
    };
    SimulatedLa.prototype.generateLaData = function (channel) {
        var maxBufferSize = Math.max.apply(Math, this.bufferSizes);
        var typedArray = new Int16Array(maxBufferSize);
        for (var i = 0; i < typedArray.length; i++) {
            typedArray[i] = i;
        }
        return {
            command: "read",
            statusCode: 0,
            wait: 0,
            binaryLength: 2 * typedArray.length,
            binaryOffset: null,
            acqCount: 3,
            bitmask: this.bitmasks[channel],
            actualSampleFreq: this.sampleFreqs[channel],
            y: typedArray,
            pointOfInterest: this.bufferSizes[channel] / 2,
            triggerIndex: this.bufferSizes[channel] / 2,
            actualTriggerDelay: this.triggerDelays[channel]
        };
    };
    return SimulatedLa;
}());
exports.SimulatedLa = SimulatedLa;
//# sourceMappingURL=simulated-la.js.map