"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Observable_1 = require("rxjs/Observable");
var generic_transport_1 = require("./generic-transport");
var simulated_device_1 = require("../simulated-device/simulated-device");
var LocalTransport = /** @class */ (function (_super) {
    __extends(LocalTransport, _super);
    function LocalTransport(deviceEnumeration) {
        var _this = _super.call(this) || this;
        _this.streamState = {
            mode: 'off',
            remainingSamples: 0
        };
        _this.simulatedDevice = new simulated_device_1.SimulatedDevice(deviceEnumeration);
        return _this;
    }
    LocalTransport.prototype.getUri = function () {
        return this.rootUri;
    };
    LocalTransport.prototype.setTimeout = function (newTimeout) { };
    LocalTransport.prototype.getRequest = function (requestUrl, timeout) {
        return Observable_1.Observable.create(function (observer) {
            observer.error('Local transport does not support get requests');
        });
    };
    //Data transmission wrapper to avoid duplicate code. 
    LocalTransport.prototype.writeRead = function (endpoint, sendData, dataType) {
        return this.writeReadHelper(this.rootUri, endpoint, sendData, dataType);
    };
    LocalTransport.prototype.writeReadHelper = function (rootUri, endpoint, sendData, dataType) {
        var _this = this;
        var body = sendData;
        return Observable_1.Observable.create(function (observer) {
            _this.simulatedDevice.send(body).subscribe(function (data) {
                observer.next(data);
            }, function (err) {
                observer.error(err);
            }, function () {
            });
        });
    };
    //Stream via back to back xhr calls
    LocalTransport.prototype.streamFrom = function (endpoint, sendData, dataType, delay) {
        var _this = this;
        if (delay === void 0) { delay = 0; }
        this.streamState.mode = 'continuous';
        return Observable_1.Observable.create(function (observer) {
            var i = 0;
            var getData = function (writeReadHelper, streamState, rootUri, endpoint, sendData, delay) {
                writeReadHelper(rootUri, endpoint, sendData).subscribe(function (data) {
                    observer.next(data);
                }, function (err) {
                    observer.error(err);
                    return;
                }, function () {
                    i++;
                    if (streamState.mode == 'continuous') {
                        //Wrap getData in anaonymous function to allow passing parameters to setTimeout handler
                        setTimeout(function () {
                            getData(writeReadHelper, streamState, rootUri, endpoint, sendData, delay);
                        }, delay);
                    }
                    else {
                        observer.complete();
                    }
                });
            };
            getData(_this.writeReadHelper, _this.streamState, _this.rootUri, endpoint, sendData, delay);
        });
    };
    //Sets stream to off
    LocalTransport.prototype.stopStream = function () {
        this.streamState.mode = 'off';
    };
    //Get transport type
    LocalTransport.prototype.getType = function () {
        return 'local';
    };
    return LocalTransport;
}(generic_transport_1.GenericTransport));
exports.LocalTransport = LocalTransport;
//# sourceMappingURL=local-transport.js.map