"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var http_transport_1 = require("./http-transport");
var local_transport_1 = require("./local-transport");
var TransportContainer = /** @class */ (function () {
    function TransportContainer(_uri, httpTimeout) {
        this.transport = new http_transport_1.HttpTransport(_uri, httpTimeout);
        this.httpTimeout = httpTimeout;
    }
    //Set transport uri
    TransportContainer.prototype.setUri = function (uri) {
        this.transport.setUri(uri);
    };
    TransportContainer.prototype.getUri = function () {
        return this.transport.getUri();
    };
    //Get request on the specified url
    TransportContainer.prototype.getRequest = function (requestUrl, timeout) {
        return this.transport.getRequest(requestUrl, timeout);
    };
    //Call writeRead on transport component
    TransportContainer.prototype.writeRead = function (endpoint, sendData, dataType, timeoutOverride) {
        return this.transport.writeRead(endpoint, sendData, dataType, timeoutOverride);
    };
    //Call streamFrom on stransport component
    TransportContainer.prototype.streamFrom = function (endpoint, sendData, dataType, delay) {
        if (delay === void 0) { delay = 0; }
        return this.transport.streamFrom(endpoint, sendData, dataType, delay);
    };
    //Stop all transport streams
    TransportContainer.prototype.stopStream = function () {
        this.transport.stopStream();
    };
    //Get type of transport service
    TransportContainer.prototype.getType = function () {
        return this.transport.getType();
    };
    TransportContainer.prototype.setHttpTransport = function (uri) {
        delete this.transport;
        this.transport = new http_transport_1.HttpTransport(uri, this.httpTimeout);
    };
    TransportContainer.prototype.setLocalTransport = function (deviceEnumeration) {
        delete this.transport;
        this.transport = new local_transport_1.LocalTransport(deviceEnumeration);
    };
    TransportContainer.prototype.setHttpTimeout = function (newHttpTimeout) {
        this.httpTimeout = newHttpTimeout;
        this.transport.setTimeout(newHttpTimeout);
    };
    return TransportContainer;
}());
exports.TransportContainer = TransportContainer;
//# sourceMappingURL=transport-container.js.map